#!/usr/bin/env python
import sys, os, re, time, string, thread, commands, glob
from xml.dom.minidom import parseString
from SOAPpy import WSDL
import utils
import gps_airs_match

#soap server endpoints on gen-dev
genesisWsdl='http://gen-dev.jpl.nasa.gov:8080/genesis/GenesisService.wsdl'
L2GPSDataWsdl='http://gen-dev.jpl.nasa.gov:8080/genesis/wsdl/L2GPSData.wsdl'

#output dir
outputdir='/tmp/outputdir'
if not os.path.isdir(outputdir):
	os.mkdir(outputdir)
	
#set input variables
startdatetime='2003-01-03 00:00:00'
enddatetime='2003-01-03 23:59:59'
north=90
south=-90
west=-180
east=180
toltime=7200
tolloc=200
priority=1
retrievaltypemax=10
landfracmin=0.0
landfracmax=0.1
getWholeSwath=1

#output files
filehead="matchups"
filename=outputdir+'/'+filehead+'.nc'
countfilename=outputdir+'/'+filehead+'.count'

#get soap proxy object
genesisEndpoint=WSDL.Proxy(genesisWsdl)
L2GPSDataEndpoint=WSDL.Proxy(L2GPSDataWsdl)

#get AIRS metadata info for date range
airsXml=genesisEndpoint.getAirsMetadataByDateRange(startdatetime,enddatetime)

#get count of AIRS granules found
airsCount=utils.getTagCountFromXml(airsXml,'granuleid')
print "Got %i AIRS granules." % airsCount

#get AIRS granule info from met file
airsMetadataXml=genesisEndpoint.getAirsMetadataFromMetFile(airsXml)

#get airs metadata dict
airsMetadataDict=utils.getAirsMetadataDictFromXml(airsMetadataXml)

#print airsMetadataDict

######################################
#get GPS metadata info for date range
######################################
gpsXml=L2GPSDataEndpoint.findByTimeAndLocationBox(startdatetime,enddatetime,north,south,west,east)

#get count of GPS datasetids found
gpsCount=utils.getTagCountFromXml(gpsXml,'datasetid')
print "Got %i GPS datasets." % gpsCount

#get gps metadata dict
gpsMetadataDict=utils.getGpsMetadataDictFromXml(gpsXml)
#print gpsMetadataDict

#coarse matchup
(coarseMatch,airsIdCum)=gps_airs_match.coarseMatchup(gpsMetadataDict,airsMetadataDict,toltime,tolloc)
#print "coarseMatch is:",coarseMatch
#print "airsIdCum is:",airsIdCum

#create xml of sorted gpsids
sortedGpsXml=utils.getSortedGpsDatasetidsXml(coarseMatch)

#get GPS generic grid xml
gpsGenGridXml=genesisEndpoint.getGpsGenericGridFromDatasetids(sortedGpsXml)
#print gpsGenGridXml

#get GPS generic grid, call urllib to download to local
gpsFile=outputdir+'/'+filehead+'_gps.nc'
utils.getGpsNetcdf(gpsGenGridXml,gpsFile)

#basically checks if the file is already here, if not,
#uses urllib to download to the local dir.  It then
#replaces the dict key with the local file path instead
#of the url.
localGranuleInfo=utils.getLocalAirsGranuleFiles(airsIdCum,outputdir+'/airsdata')
#print localGranuleInfo

#findGpsScanMatch
(gpsScanMatch,matchNetcdf,swathFiles)=gps_airs_match.findGpsScanMatch(coarseMatch,gpsFile,localGranuleInfo,toltime,tolloc,priority,retrievaltypemax,landfracmin,landfracmax,filename,getWholeSwath)

print gpsScanMatch,matchNetcdf,swathFiles
