#from ZODB import FileStorage, DB
import sys, os, re, time, glob, commands, pickle, urllib, string, shutil
import xmlConverter
from xml.dom.minidom import parseString

def getTagCountFromXml(xml,tag):
    domObj=parseString(xml)
    tagElements=domObj.getElementsByTagName(tag)
    tags=[tagElement.firstChild.data for tagElement in tagElements]
                                                     
    return len(tags)

def getAirsMetadataDictFromXml(xml):
    domObj=parseString(xml)

    #get urls
    urlElements=domObj.getElementsByTagName("url")
    urls=[urlElement.firstChild.data for urlElement in urlElements]
                                                                                                                              
    #get four bounds
    northElements=domObj.getElementsByTagName("northbound")
    norths=[northElement.firstChild.data for northElement in northElements]
    southElements=domObj.getElementsByTagName("southbound")
    souths=[southElement.firstChild.data for southElement in southElements]
    westElements=domObj.getElementsByTagName("westbound")
    wests=[westElement.firstChild.data for westElement in westElements]
    eastElements=domObj.getElementsByTagName("eastbound")
    easts=[eastElement.firstChild.data for eastElement in eastElements]
                                                                                                                              
    #get start/end times
    starttimeElements=domObj.getElementsByTagName("starttime")
    starttimes=[starttimeElement.firstChild.data for starttimeElement in starttimeElements]
    endtimeElements=domObj.getElementsByTagName("endtime")
    endtimes=[endtimeElement.firstChild.data for endtimeElement in endtimeElements]
                                                                                                                              
    #build AIRS metadata dict
    airsMetadataDict={}
    for index in range(len(urls)):
        try:
            north=float(norths[index])
            south=float(souths[index])
            west=float(wests[index])
            east=float(easts[index])
            starttime=starttimes[index]
            endtime=endtimes[index]
            metadata=[starttime,endtime,west,east,south,north]
            airsMetadataDict[urls[index]]=metadata
        except Exception, e:
            print "Got exception: %s" % e

    return airsMetadataDict

def getGpsMetadataDictFromXml(xml):
    domObj=parseString(xml)

    #get datasetids
    datasetidElements=domObj.getElementsByTagName("datasetid")
    datasetids=[datasetidElement.firstChild.data for datasetidElement in datasetidElements]

    #get lat/lon
    latElements=domObj.getElementsByTagName("latitude")
    lats=[latElement.firstChild.data for latElement in latElements]
    lonElements=domObj.getElementsByTagName("longitude")
    lons=[lonElement.firstChild.data for lonElement in lonElements]

    #get start/end dates/times
    startdateElements=domObj.getElementsByTagName("rangebeginningdate")
    startdates=[startdateElement.firstChild.data for startdateElement in startdateElements]
    enddateElements=domObj.getElementsByTagName("rangeendingdate")
    enddates=[enddateElement.firstChild.data for enddateElement in enddateElements]
    starttimeElements=domObj.getElementsByTagName("rangebeginningtime")
    starttimes=[starttimeElement.firstChild.data for starttimeElement in starttimeElements]
    endtimeElements=domObj.getElementsByTagName("rangeendingtime")
    endtimes=[endtimeElement.firstChild.data for endtimeElement in endtimeElements]

    #build gps metadata dict
    gpsMetadataDict={}
    for index in range(len(datasetids)):
        try:
            lat=float(lats[index])
            lon=float(lons[index])
            startdatetime="%s %s" % (startdates[index],starttimes[index])
            enddatetime="%s %s" % (enddates[index],endtimes[index])
            metadata=[startdatetime,enddatetime,lat,lon]
            gpsMetadataDict[datasetids[index]]=metadata
        except Exception, e:
            print "Got exception: %s" % e

    return gpsMetadataDict

def getSortedGpsDatasetidsXml(gpsDict):
    gpsIds=gpsDict.keys()
    gpsIds.sort()
    gpsDatasetidsXml=xmlConverter.simpleList2Xml(gpsIds,'ResultSet','datasetid')
    return gpsDatasetidsXml

def getGpsNetcdf(xml,localFile):

    if os.path.isfile(localFile):
        return localFile
    else:
        try:
            domObj=parseString(xml)
            urlElements=domObj.getElementsByTagName("url")
            url=str(urlElements[0].firstChild.data) 
            (gpsNetcdf,headers)=urllib.urlretrieve(url,filename=localFile) 
            os.chmod(localFile,0666)
        except Exception, e:
            raise RuntimeError, "Got exception: %s" % e

        return gpsNetcdf

def getLocalAirsGranuleFiles(airsUrlList,localDataDir):

    #local granuleInfo dict
    airsFilesList=[]

    #loop over granules
    for url in airsUrlList:

        #localfile
        localFile=str(re.sub(r'^http://.*/airs/L2/\d{8}/airx2ret',localDataDir,url))

        #if local file exists, don't download.
        #otherwise do
        if os.path.isfile(localFile):
            pass
        else:
            print "downloading %s..." % url,

            #download
            (gpsNetcdf,headers)=urllib.urlretrieve(url,filename=localFile) 

            print "done."

        #set local granule info
        airsFilesList.append(localFile)

    return airsFilesList
